/*
    radialIndicator.js v 1.0.0
    Author: Sudhanshu Yadav
    Copyright (c) 2015 Sudhanshu Yadav - ignitersworld.com , released under the MIT license.
    Demo on: ignitersworld.com/lab/radialIndicator.html
*/

;(function ($, window, document) {
    "use strict";
    //circumfence and quart value to start bar from top
    var circ = Math.PI * 2,
        quart = Math.PI / 2;
    var DTD = '<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">'
        , XMLNS = 'http://www.w3.org/2000/svg'
        ;

    var create = function(target_, postion_, left_, top_, width_, height_, bgColor_){
            function makeSVG(width, height){
                var svgNode = document.createElementNS(XMLNS, 'svg');
                svgNode.setAttribute('version', '1.1');
                svgNode.setAttribute("xmlns", XMLNS);
                svgNode.setAttribute('width', width);
                svgNode.setAttribute("height", height);
                return svgNode;
            }
            width_ = width_ ? width_ : target_.width();
            height_ = height_ ? height_ : target_.height();
            var dom = $(makeSVG(width_, height_)).css({
                "float":"left",
                "position": postion_ ? postion_ : ABSOLUTE,
                "left": left_ ? left_ : 0,
                "top": top_ ? top_ : 0,
                "width": width_,
                "height": height_,
                "background-color": bgColor_ ? bgColor_ : "transparent"
            }).appendTo(target_).get(0);
            return dom;
        },
        createGroup = function(target_, class_, strokeColor_, strokeWidth_, fillColor_){
            function makeSVGGroup(width, height){
                var svgGroupNode = document.createElementNS(XMLNS, 'g');
                return svgGroupNode;
            }
            return $(makeSVGGroup()).attr("class", class_).appendTo(target_).get(0);
        };
    var CE = {
        createElement : function( type, attr ) {
            var ele = document.createElementNS(XMLNS, type);
            this.attr(ele, attr);
            return ele;
        },
        attr : function( ele, obj ) {
            for( var i in obj ) {
                ele.setAttribute(i, obj[i]);
            }
        },
        circle : function( ptCenter, radius, strokeColor, strokeWidth, fillColor, alpha ) {
            var circle = this.createElement('circle');
            var attr = {
                "cx" : ptCenter.x,
                "cy" : ptCenter.y,
                "r" : radius
            };
            if (fillColor) {
                attr["fill"] = fillColor;
            }
            if (strokeWidth > 0) {
                attr["stroke"] = strokeColor;
                attr["stroke-width"] = strokeWidth;
            }
            if(alpha){
                attr["opacity"] = alpha;
            }
            this.attr(circle, attr);
            return circle;
        },
        path : function( d, strokeWidth, strokeColor, fillColor ) {
            var path = this.createElement('path');
            var attr = {
                "d" : d,
                "stroke" : strokeColor,
                "stroke-width" : strokeWidth,
                "fill" : fillColor,
                "stroke-linejoin" : "round",
                "stroke-linecap" : "round"
            };
            this.attr(path,attr);
            return path;
        },
        text : function( text_, x_, y_ ) {
            var text = this.createElement("text");
            $(text).text(text_);
            var attr = {
                x : x_,
                y : y_
            };
            this.attr(text, attr);
            return text;
        }
    };
    var drawText = function( ctx, text_, x, y ) {
        var text = CE.text( text_, x, y );
        ctx.appendChild(text);
        return text;
    };
    var drawArcP = function( b, e, w, r, c ) {
        if (e < b){
            var tmp = b;
            b = e;
            e = tmp;
        }
        var fr = r-w;
        var sX1 = c.x + fr * Math.cos(b);
        var sY1 = c.y - fr * Math.sin(b);
        var sX2 = c.x + r * Math.cos(b);
        var sY2 = c.y - r * Math.sin(b);

        var eX1 = c.x + r * Math.cos(e);
        var eY1 = c.y - r * Math.sin(e);
        var eX2 = c.x + fr * Math.cos(e);
        var eY2 = c.y - fr * Math.sin(e);
        var path = "";
        path += "M" + sX1 + " " + sY1 + " ";
        path += "L" + sX2 + " " + sY2 + " ";
        if (e - b >= Math.PI){
            path += "A" + r + " " + r + " 0 1 0 " + eX1 + " " + eY1 + " ";
        }else{
            path += " A " + r + " " + r + " 0 0 0 " + eX1 + " " + eY1 + " ";
        }
        path += "L" + eX2 + " " + eY2 + " ";
        if(e - b >= Math.PI) {
            path += "A" + fr + " " + fr + " 0 1 1  " + sX1 + " " + sY1 + " ";
        }else{
            path += "A" + fr + " " + fr + " 0 0 1 " + sX1 + " " + sY1 + " ";
        }
        path += "z ";
        return path;
    };
    var drawArc = function( svgDoc, ptCenter, radius, beginArc, endArc, strokeColor, strokeWidth, fillColor, arcWidth ) {
        
        var path = CE.path("", strokeWidth, strokeColor, fillColor);
        svgDoc.appendChild(path);
    };
    var drawCircle = function( svgDoc, ptCenter, radius, strokeColor, strokeWidth, fillColor, alpha ) {
        var circle = CE.circle(ptCenter, radius, strokeColor, strokeWidth, fillColor, alpha);
        svgDoc.appendChild(circle);
    };
    //function to convert hex to rgb

    function hexToRgb(hex) {
        // Expand shorthand form (e.g. "03F") to full form (e.g. "0033FF")
        var shorthandRegex = /^#?([a-f\d])([a-f\d])([a-f\d])$/i;
        hex = hex.replace(shorthandRegex, function (m, r, g, b) {
            return r + r + g + g + b + b;
        });

        var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
        return result ? [parseInt(result[1], 16), parseInt(result[2], 16), parseInt(result[3], 16)] : null;
    }

    function getPropVal(curShift, perShift, bottomRange, topRange) {
        return Math.round(bottomRange + ((topRange - bottomRange) * curShift / perShift));
    }


    //function to get current color in case of
    function getCurrentColor(curPer, bottomVal, topVal, bottomColor, topColor) {
        var rgbAryTop = topColor.indexOf('#') != -1 ? hexToRgb(topColor) : topColor.match(/\d+/g),
            rgbAryBottom = bottomColor.indexOf('#') != -1 ? hexToRgb(bottomColor) : bottomColor.match(/\d+/g),
            perShift = topVal - bottomVal,
            curShift = curPer - bottomVal;

        if (!rgbAryTop || !rgbAryBottom) return null;

        return 'rgb(' + getPropVal(curShift, perShift, rgbAryBottom[0], rgbAryTop[0]) + ',' + getPropVal(curShift, perShift, rgbAryBottom[1], rgbAryTop[1]) + ',' + getPropVal(curShift, perShift, rgbAryBottom[2], rgbAryTop[2]) + ')';
    }

    //to merge object
    function merge() {
        var arg = arguments,
            target = arg[0];
        for (var i = 1, ln = arg.length; i < ln; i++) {
            var obj = arg[i];
            for (var k in obj) {
                if (obj.hasOwnProperty(k)) {
                    target[k] = obj[k];
                }
            }
        }
        return target;
    }

    //function to apply formatting on number depending on parameter
    function formatter(pattern) {
        return function (num) {
            if(!pattern) return num.toString();
            num = num || 0
            var numRev = num.toString().split('').reverse(),
                output = pattern.split("").reverse(),
                i = 0,
                lastHashReplaced = 0;

            //changes hash with numbers
            for (var ln = output.length; i < ln; i++) {
                if (!numRev.length) break;
                if (output[i] == "#") {
                    lastHashReplaced = i;
                    output[i] = numRev.shift();
                }
            }

            //add overflowing numbers before prefix
            output.splice(lastHashReplaced+1, output.lastIndexOf('#') - lastHashReplaced, numRev.reverse().join(""));

            return output.reverse().join('');
        }
    }


    //circle bar class
    function Indicator(container, indOption) {
        indOption = indOption || {};
        indOption = merge({}, radialIndicator.defaults, indOption);

        this.indOption = indOption;

        //create a queryselector if a selector string is passed in container
        if (typeof container == "string")
            container = document.querySelector(container);

        //get the first element if container is a node list
        if (container.length)
            container = container[0];

        this.container = container;

        //create a canvas element
        // var canElm = document.createElement("canvas");
        // container.appendChild(canElm);
        // console.info(container);
        var chartPlace = $('<div class="chartPlace"></div>').appendTo(container);
        this.targetHolder = create(chartPlace, "relative", 0, 0, $(container).width(), $(container).height());
        var graphGroup = createGroup(this.targetHolder, "ChartGraph", "none", 0, "none");

        this.canElm = graphGroup; // dom object where drawing will happen

        this.ctx = graphGroup; //get 2d canvas context

        //add intial value
        this.current_value = indOption.initValue || indOption.minValue || 0;

    }


    Indicator.prototype = {
        constructor: radialIndicator,
        init: function () {
            var indOption = this.indOption,
                canElm = this.canElm,
                ctx = this.ctx,
                dim = (indOption.radius + indOption.barWidth) * 2, //elm width and height
                center = dim / 2; //center point in both x and y axis


            //create a formatter function
            this.formatter = typeof indOption.format == "function" ? indOption.format : formatter(indOption.format);

            //maximum text length;
            this.maxLength = indOption.percentage ? 4 : this.formatter(indOption.maxValue).length;

            canElm.width = dim;
            canElm.height = dim;

            //draw a grey circle
            ctx.strokeStyle = indOption.barBgColor; //background circle color
            ctx.lineWidth = indOption.barWidth;
            // ctx.beginPath();
            //svgDoc, ptCenter, radius, strokeColor, strokeWidth, fillColor, alpha
            drawCircle(ctx, {x:center,y:center}, indOption.radius,  indOption.barBgColor, indOption.barWidth, "#fff", 0);
            // ctx.stroke();

            //store the image data after grey circle draw
            // this.imgData = ctx.getImageData(0, 0, dim, dim);

            //put the initial value if defined
            this.value(this.current_value);

            return this;
        },
        //update the value of indicator without animation
        value: function (val) {
            //return the val if val is not provided
            if (val === undefined || isNaN(val)) {
                return this.current_value;
            }

            val = parseInt(val);

            var ctx = createGroup(this.targetHolder, "ValueChartGraph", "none", 0, "none"),
                indOption = this.indOption,
                curColor = indOption.barColor,
                dim = (indOption.radius + indOption.barWidth) * 2,
                minVal = indOption.minValue,
                maxVal = indOption.maxValue,
                center = dim / 2;

            //limit the val in range of 0 to 100
            val = val < minVal ? minVal : val > maxVal ? maxVal : val;

            var perVal = Math.round(((val - minVal) * 100 / (maxVal - minVal)) * 100) / 100, //percentage value tp two decimal precision
                dispVal = indOption.percentage ? perVal + '%' : this.formatter(val); //formatted value
            if(indOption.txt != '') {
                dispVal = indOption.txt;
            }

            //save val on object
            this.current_value = val;


            //draw the bg circle
            // ctx.putImageData(this.imgData, 0, 0);

            //get current color if color range is set
            if (typeof curColor == "object") {
                var range = Object.keys(curColor);

                for (var i = 1, ln = range.length; i < ln; i++) {
                    var bottomVal = range[i - 1],
                        topVal = range[i],
                        bottomColor = curColor[bottomVal],
                        topColor = curColor[topVal],
                        newColor = val == bottomVal ? bottomColor : val == topVal ? topColor : val > bottomVal && val < topVal ? indOption.interpolate ? getCurrentColor(val, bottomVal, topVal, bottomColor, topColor) : topColor : false;

                    if (newColor != false) {
                        curColor = newColor;
                        break;
                    }
                }
            }

            //draw th circle value
            ctx.strokeStyle = curColor;
            //add linecap if value setted on options
            if (indOption.roundCorner) ctx.lineCap = "round";
            var svg = d3.select("#chart")
                .append("svg")
                .attr("width", dim)
                .attr("height", dim)
                .append("g")
                .attr("transform", "translate("+(dim/2)+","+(dim/2)+")");

            var arc = d3.svg.arc()
                .innerRadius(indOption.radius-indOption.barWidth+3)
                .outerRadius(indOption.radius+3)
                .startAngle( 0 )
                .endAngle(((perVal / 100)*2) * Math.PI);


            // ctx.beginPath();
            drawArc(ctx, {x:center, y : center}, indOption.radius, -(quart), ((circ) * perVal / 100) - quart, '', '', curColor);
            $(".ValueChartGraph").find("path").attr("transform", "translate(36,36)").attr('d', arc);
            // ctx.stroke();

            //add percentage text
            if (indOption.displayNumber) {
                var tctx = createGroup(this.targetHolder, "textChartGraph", "none", 0, "none");
                var weight = indOption.fontWeight,
                    fontSize = indOption.fontSize || (dim / (this.maxLength - (Math.floor(this.maxLength*1.4/4)-1)));
                var div = $('<label>'+dispVal+'</label>').css({
                    "font-size" : fontSize,
                    "fon-family" : indOption.fontFamily
                }).appendTo("body");
                var width = $(div).width();
                var height = $(div).height();
                // console.info(height)
                // var cFont = indOption.fontFamily || cFont[cFont.length - 1];

                // console.info(fontSize);
                var t = drawText(tctx,dispVal,(dim-width)*0.5, center+(height)*0.25);
                $(t).attr({
                    "fill" : indOption.fontColor || curColor,
                    "font-size" : fontSize,
                    "font-family" : indOption.fontFamily
                });
                $(div).remove();
                // ctx.fillStyle = indOption.fontColor || curColor;
                // ctx.font = weight +" "+ fontSize + "px " + cFont;
                // ctx.textAlign = "center";
                // ctx.textBaseline = 'middle';
                // ctx.fillText(dispVal, center, center);
            }

            return this;
        },
        //animate progressbar to the value
        animate: function (val) {

            var indOption = this.indOption,
                counter = this.current_value || indOption.minValue,
                self = this,
                incBy = Math.ceil((indOption.maxValue - indOption.minValue) / (indOption.frameNum || (indOption.percentage ? 100 : 500))), //increment by .2% on every tick and 1% if showing as percentage
                back = val < counter;

            //clear interval function if already started
            if (this.intvFunc) clearInterval(this.intvFunc);

            this.intvFunc = setInterval(function () {

                if ((!back && counter >= val) || (back && counter <= val)) {
                    if (self.current_value == counter) {
                        clearInterval(self.intvFunc);
                        return;
                    } else {
                        counter = val;
                    }
                }

                self.value(counter); //dispaly the value

                if (counter != val) {
                    counter = counter + (back ? -incBy : incBy)
                }; //increment or decrement till counter does not reach  to value
            }, indOption.frameTime);

            return this;
        },
        //method to update options
        option: function (key, val) {
            if (val === undefined) return this.option[key];

            if (['radius', 'barWidth', 'barBgColor', 'format', 'maxValue', 'percentage'].indexOf(key) != -1) {
                this.indOption[key] = val;
                this.init().value(this.current_value);
            }
            this.indOption[key] = val;
        }

    };

    /** Initializer function **/
    function radialIndicator(container, options) {
        var progObj = new Indicator(container, options);
        progObj.init();
        return progObj;
    }

    //radial indicator defaults
    radialIndicator.defaults = {
        radius: 50, //inner radius of indicator
        barWidth: 5, //bar width
        barBgColor: '#eeeeee', //unfilled bar color
        barColor: '#99CC33', //filled bar color , can be a range also having different colors on different value like {0 : "#ccc", 50 : '#333', 100: '#000'}
        format: null, //format indicator numbers, can be a # formator ex (##,###.##) or a function
        frameTime: 10, //miliseconds to move from one frame to another
        frameNum: null, //Defines numbers of frame in indicator, defaults to 100 when showing percentage and 500 for other values
        fontColor: null, //font color
        fontFamily: null, //defines font family
        fontWeight: 'bold', //defines font weight
        fontSize : null, //define the font size of indicator number
        interpolate: true, //interpolate color between ranges
        percentage: false, //show percentage of value
        displayNumber: true, //display indicator number
        roundCorner: false, //have round corner in filled bar
        minValue: 0, //minimum value
        maxValue: 100, //maximum value
        initValue: 0 //define initial value of indicator
    };

    window.radialIndicator = radialIndicator;

    //add as a jquery plugin
    if ($) {
        $.fn.radialIndicator = function (options) {
            return this.each(function () {
                var newPCObj = radialIndicator(this, options);
                $.data(this, 'radialIndicator', newPCObj);
            });
        };
    }

}(window.jQuery, window, document, void 0));
