define("ZSYWEB.NativeAPI.MOBILE", function (require) {
    var SLICE = Array.prototype.slice;
    var MobileHybrid = {};
    MobileHybrid = {
        registerHandler: registerHandler,
        callHandler: callHandler,
        disableJavscriptAlertBoxSafetyTimeout: disableJavscriptAlertBoxSafetyTimeout,
        _fetchQueue: _fetchQueue,
        _handleMessageFromObjC: _handleMessageFromObjC,
        version : '0.0.1',
        desc : 'JavaScript hybrid framework.'
    };

    var messagingIframe;
    var sendMessageQueue = [];
    var messageHandlers = {};

    var CUSTOM_PROTOCOL_SCHEME = 'wvjbscheme';
    var QUEUE_HAS_MESSAGE = '__WVJB_QUEUE_MESSAGE__';

    var responseCallbacks = {};
    var uniqueId = 1;
    var dispatchMessagesWithTimeoutSafety = true;

    function registerHandler(handlerName, handler) {
        messageHandlers[handlerName] = handler;
    }

    function callHandler(handlerName, data, responseCallback) {
        console.info([handlerName, data]);
        if (arguments.length == 2 && typeof data == 'function') {
            responseCallback = data;
            data = null;
        }
        _doSend({
            handlerName: handlerName,
            data: data
        }, responseCallback);
    }

    function disableJavscriptAlertBoxSafetyTimeout() {
        dispatchMessagesWithTimeoutSafety = false;
    }

    function _doSend(message, responseCallback) {
        if (responseCallback) {
            var callbackId = 'cb_' + (uniqueId++) + '_' + new Date().getTime();
            responseCallbacks[callbackId] = responseCallback;
            message['callbackId'] = callbackId;
        }
        sendMessageQueue.push(message);
        messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://' + QUEUE_HAS_MESSAGE;
    }

    function _fetchQueue() {
        var messageQueueString = JSON.stringify(sendMessageQueue);
        sendMessageQueue = [];
        return messageQueueString;
    }

    function _dispatchMessageFromObjC(messageJSON) {
        if (dispatchMessagesWithTimeoutSafety) {
            setTimeout(_doDispatchMessageFromObjC);
        } else {
            _doDispatchMessageFromObjC();
        }

        function _doDispatchMessageFromObjC() {
            var message = JSON.parse(messageJSON);
            var messageHandler;
            var responseCallback;

            if (message.responseId) {
                responseCallback = responseCallbacks[message.responseId];
                if (!responseCallback) {
                    return;
                }
                responseCallback(message.responseData);
                delete responseCallbacks[message.responseId];
            } else {
                if (message.callbackId) {
                    var callbackResponseId = message.callbackId;
                    responseCallback = function(responseData) {
                        _doSend({
                            handlerName: message.handlerName,
                            responseId: callbackResponseId,
                            responseData: responseData
                        });
                    };
                }

                var handler = messageHandlers[message.handlerName];
                if (!handler) {
                    console.log("Hybrid: WARNING: no handler for message from ObjC:", message);
                } else {
                    handler(message.data, responseCallback);
                }
            }
        }
    }

    function _handleMessageFromObjC(messageJSON) {
        _dispatchMessageFromObjC(messageJSON);
    }
    var initHybrid = function() {
        messagingIframe = document.createElement('iframe');
        messagingIframe.style.display = 'none';
        messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://' + QUEUE_HAS_MESSAGE;
        document.documentElement.appendChild(messagingIframe);

        registerHandler("_disableJavascriptAlertBoxSafetyTimeout", disableJavscriptAlertBoxSafetyTimeout);

        setTimeout(_callWVJBCallbacks, 0);
    };

    function _callWVJBCallbacks() {
        var callbacks = window.WVJBCallbacks;
        delete window.WVJBCallbacks;
        if (!callbacks) {
            return;
        };
        for (var i = 0; i < callbacks.length; i++) {
            callbacks[i](MobileHybrid);
        }
    }
    function type(o) {
        return Object.prototype.toString.apply(o).slice(8, -1).toLowerCase()
    }

    function isFn(fn) {
        return type(fn) === 'function'
    }

    // function noop() {
    //     let args = SLICE.call(arguments)
    //     let callback = args.length && args[args.length - 1]

    //     return isFn(callback) ? callback(null) : null
    // }
    function inject(ns, api, handler) {
        if(arguments.length < 3) {
            handler = arguments[1]
            api = arguments[0]
            ns = ''
        }

        var names = api.split(/\./)
        var fnName = names.pop()

        if(names[0] === ns) names.shift()

        var root = ns ? (MobileHybrid[ns] || (MobileHybrid[ns] = {})) : MobileHybrid

        root = createNamespace(root, names)

        if(fnName && isFn(handler)) root[fnName] = handler;
    }
    function wrapAPI(api, wrapper) {
        if(/^web\./.test(api)) throw new Error('此处不要使用web作为根命名空间');
        inject(api, wrapper);
    }
    function createNamespace(root, names) {
        var i = 0;
        var a;
        for( var i = 0; i < names.length; i++ ) {
            a = names[i];
            if( root[a] && type(root[a]) !== 'object' ) {

            }else if( !root[a] ){
                root[a] = {};
            }
            root = root[a];
        }
//        while(a = names[i++]) {
//            if(root[a] && 'object' !== type(root[a])) throw Error(`${a} 命名空间冲突`);
//            if(!(root[a])) root[a] = {};
//            root = root[a];
//        }

        return root
    }
    MobileHybrid.initHybrid = initHybrid;
    MobileHybrid.wrapAPI = wrapAPI;
    return MobileHybrid;
});